/**
 * Copyright 2006 by jFramework.org
 */
package org.jframework.core.util;

import junit.framework.TestCase;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.Transformer;
import org.apache.log4j.BasicConfigurator;

/**
 * A base for all bean tests.
 *
 * @author mgridnev
 * @version 2.0, Dec 7, 2006
 */
public abstract class BeanTestCase<T> extends TestCase
{
    /**
     * The bean
     */
    protected T bean;

    /**
     * Sets up the test run.
     */
    protected final void setUp() throws Exception
    {
        BasicConfigurator.configure();

        bean = newBean();

        onSetUp();
    }

    /**
     * Sets up the test run.
     */
    protected void onSetUp()
    {
    }

    /**
     * Cleans up after test run.
     */
    protected final void tearDown() throws Exception
    {
        onTearDown();

        BasicConfigurator.resetConfiguration();
    }

    /**
     * Cleans up after test run.
     */
    protected void onTearDown()
    {
    }

    /**
     * Tests the bean
     */
    @SuppressWarnings("unchecked")
    public final void testBean()
    {
        final Object bean = this.bean;

        final List<Check> checks = getPropertyChecks();
        final int cChecks = checks.size();

        // Populate with initial values
        List listExpectedValues = new ArrayList(checks);
        CollectionUtils.transform(listExpectedValues, new Transformer()
        {
            public Object transform(Object input)
            {
                return ((Check) input).initial;
            }
        });

        // Check initial values
        assertProperties("Initial values", bean, checks, listExpectedValues);

        // Check updated values
        for (int i = 0; i < cChecks; i++) {
            final Check check = checks.get(i);

            if (check.updated != Check.UNDEFINED) {
                listExpectedValues.set(i, check.updated);
                BeanUtils.setProperty(bean, check.property, check.updated);
            }

            assertProperties("Initial to updated", bean, checks, listExpectedValues);
        }

        // Go back to initial values
        for (int i = 0; i < cChecks; i++) {
            final Check check = checks.get(i);

            if (check.updated != Check.UNDEFINED) {
                listExpectedValues.set(i, check.initial);
                BeanUtils.setProperty(bean, check.property, check.initial);
            }

            assertProperties("Updated to initial", bean, checks, listExpectedValues);
        }
    }

    /**
     * Assert property value.
     *
     * @param message
     * @param bean       the bean
     * @param checks   an array of property checks
     * @param values a list of property values
     */
    private void assertProperties(String message, Object bean, List<Check> checks, List values)
    {
        final int cChecks = checks.size();
        for (int i = 0; i < cChecks; i++) {
            Check check = checks.get(i);
            assertEquals(message + ": property = " + check.property, values.get(i), BeanUtils.getProperty(bean, check.property));
        }
    }

    /**
     * Creates a new bean
     */
    protected abstract T newBean();

    /**
     * Returns property checks
     */
    protected abstract List<Check> getPropertyChecks();

    /**
     * Property check description
     */
    public static class Check
    {
        /**
         * Undefined value
         */
        public static final Object UNDEFINED = new Object();

        /**
         * Property name
         */
        private String property;

        /**
         * Initial property value
         */
        private Object initial;

        /**
         * Updated property value
         */
        private Object updated;

        /**
         * Creates a read-write property check
         *
         * @param name   property name
         * @param init   initial value
         * @param update value to update to
         * @return new property check
         */
        public static Check newWritable(String name, Object init, Object update)
        {
            return new Check(name, init, update);
        }

        /**
         * Creates a read-only property check
         *
         * @param name property name
         * @param init initial value
         * @return new property check
         */
        public static Check newReadOnly(String name, Object init)
        {
            return new Check(name, init, UNDEFINED);
        }

        /**
         * Creates a property check
         *
         * @param name   property name
         * @param init   initial value
         * @param update value to updated to, specify {@link #UNDEFINED} to skip the check
         */
        private Check(String name, Object init, Object update)
        {
            this.property = name;
            this.initial = init;
            this.updated = update;
        }
    }

}
