/**
 * Copyright 2006 by jFramework.org.
 */
package org.jframework.core.util;

import junit.framework.TestCase;

import java.io.UnsupportedEncodingException;
import java.math.BigInteger;

import org.apache.log4j.BasicConfigurator;

/**
 * Number utilities test.
 *
 * @author mgridnev
 * @version 1.0, Aug 11, 2006
 */
public class NumberUtilsTest extends TestCase
{
    /**
     * Sets up test run
     */
    protected void setUp() throws Exception
    {
        BasicConfigurator.configure();
    }

    /**
     * Cleans up after test run
     */
    protected void tearDown() throws Exception
    {
        BasicConfigurator.resetConfiguration();
    }

    /**
     * Tests unsigned long conversion
     */
    public void testUnsignedLong()
    {
        // Test valid value

        // Create a number larger than the max signed long value
        BigInteger numberOriginal = BigInteger.valueOf(Long.MAX_VALUE);
        numberOriginal = numberOriginal.add(BigInteger.valueOf(128));

        final long signed = NumberUtils.parseUnsignedLong(numberOriginal.toString(10), 10);
        assertEquals(signed, NumberUtils.parseUnsignedLong(numberOriginal.toString(10))); // decimal version

        // Restore the number
        // IMPORTANT: Must do it in hex so sign is ignored
        final BigInteger numberRestored = new BigInteger(NumberUtils.toUnsignedLongString(signed, 10));
        assertEquals(numberRestored, new BigInteger(NumberUtils.toUnsignedLongString(signed))); // decimal version

        assertEquals(numberOriginal, numberRestored);

        // Test invalid values

        // Negative
        try {
            NumberUtils.parseUnsignedLong("-1", 10);
            fail();
        } catch (NumberFormatException e) {
        }

        // Too large
        try {
            NumberUtils.parseUnsignedLong(BigInteger.valueOf(Long.MAX_VALUE).multiply(BigInteger.TEN).toString(), 10);
            fail();
        } catch (NumberFormatException e) {
        }
    }

    /**
     * Tests unsigned int conversion
     */
    public void testUnsignedInt()
    {
        // Test valid value

        // Create a number larger than the max signed int value
        BigInteger numberOriginal = BigInteger.valueOf(Integer.MAX_VALUE);
        numberOriginal = numberOriginal.add(BigInteger.valueOf(128));

        final int signed = NumberUtils.parseUnsignedInt(numberOriginal.toString(10), 10);
        assertEquals(signed, NumberUtils.parseUnsignedInt(numberOriginal.toString(10))); // decimal version

        // Restore the number
        // IMPORTANT: Must do it in hex so sign is ignored
        final BigInteger numberRestored = new BigInteger(NumberUtils.toUnsignedIntString(signed, 10));
        assertEquals(numberRestored, new BigInteger(NumberUtils.toUnsignedIntString(signed))); // decimal version

        assertEquals(numberOriginal, numberRestored);

        // Test invalid values

        // Negative
        try {
            NumberUtils.parseUnsignedInt("-1", 10);
            fail();
        } catch (NumberFormatException e) {
        }

        // Too large
        try {
            NumberUtils.parseUnsignedInt(BigInteger.valueOf(Integer.MAX_VALUE).multiply(BigInteger.TEN).toString(), 10);
            fail();
        } catch (NumberFormatException e) {
        }
    }


    /**
     * Tests byte array conversion
     */
    public void testByteArray()
    {
        // Valid array
        final byte[] rgBytes = new byte[255];
        for (int i = 0; i < rgBytes.length; i++) {
            rgBytes[i] = (byte) i;
        }

        final String string = NumberUtils.toByteArrayString(rgBytes);
        assertEquals(rgBytes.length * 2, string.length());
        assertEquals("000102030405060708090A0B0C0D0E0F10", string.substring(0, 34).toUpperCase());

        byte[] rgResult = NumberUtils.parseByteArrayString(string);
        assertEquals(rgBytes.length, rgResult.length);
        for (int i = 0; i < rgBytes.length; i++) {
            assertEquals("Index " + i, rgBytes[i], rgResult[i]);
        }

        // Empty array
        assertEquals("", NumberUtils.toByteArrayString(new byte[0]));

        rgResult = NumberUtils.parseByteArrayString("");
        assertEquals(0, rgResult.length);

        // Invalid chars
        try {
            NumberUtils.parseByteArrayString("0x");
            fail();
        } catch (NumberFormatException e) {
        }

        // Invalid length
        try {
            NumberUtils.parseByteArrayString("01a");
            fail();
        } catch (NumberFormatException e) {
        }
    }

    /**
     * Tests MD5 hash calculation
     */
    public void testMD5() throws UnsupportedEncodingException
    {
        final String input = "Hello world!";
        final String MD5 = "86fb269d190d2c85f6e0468ceca42a20";

        final byte[] rgMD5 = NumberUtils.MD5(input.getBytes("UTF-8"));

        assertEquals(MD5.toUpperCase(), NumberUtils.toByteArrayString(rgMD5).toUpperCase());
    }
}
